#!/bin/perl
use strict; 

use constant GNUPLOT => '/usr/bin/gnuplot';
#use constant GNUPLOT => '/usr/freeware/bin/gnuplot';

# Print a helpful message if the user provides no input file.
if (!@ARGV) { 
        print "usage: gnuphase.pl num_files [file1 file2 ...] case[0-6] [LOD|SC] cutoff [x1] [x2] [y1] [y2]  \n";
        print "cases:     0=GLOBAL\n";
        print "           1=LOCAL_DIAG_VITERBI 2=LOCAL_DIAG_FORWARD\n";
        print "           3=LOCAL_SEMI_VITERBI 4=LOCAL_SEMI_FORWARD\n";
        print "           5=LOCAL_FULL_VITERBI 6=LOCAL_FULL_FORWARD\n\n";
        exit;
}

my $demos   = "../Demos";
my $scripts = "../scripts";
my $output  = "../Demos";
my $tag;

my $separate;
my $identical;

#INPUT num_file
my $num_files = shift;

#INPUT file
my $num = 0;
my @file;
while($num < $num_files) {
    $file[$num] = shift;
    $num++;
}

#INPUT algorithm (0-6)
my $case = shift;
if   ($case==0) { $tag = "     [GLOBAL]";        }
elsif($case==1) { $tag = "     [LOCAL_DIAG_VITERBI]"; }
elsif($case==2) { $tag = "     [LOCAL_DIAG_FORWARD]"; }
elsif($case==3) { $tag = "     [LOCAL_SEMI_VITERBI]"; }
elsif($case==4) { $tag = "     [LOCAL_SEMI_FORWARD]"; }
elsif($case==5) { $tag = "     [LOCAL_FULL_VITERBI]"; }
elsif($case==6) { $tag = "     [LOCAL_FULL_FORWARD]"; }

#INPUT LOD or SC
my $logodds = shift;
if ($logodds !~ /LOD/ && $logodds !~ /SC/) { die "LOD or SC only\n";}

#if you want to plot a cutoff
my $cutoff  = shift;
my $cutoff2 = 1.4142*$cutoff;

#INPUT coordinates [x1,x2] [y1,y2]
my $x1 = shift;
my $x2 = shift;
my $y1 = shift;
my $y2 = shift;

# to print a line in the plot
my $cod0;
my $cod1;
my $rna0;
my $rna1;
my $m;
my $a;
my $xa;
my $xb;

if (@ARGV) 
{ 
    $cod0 = shift; $rna0 = shift; 
    $cod1 = shift; $rna1 = shift; 

    if ($cod0 != $cod1) 
    { 
	$m = ($rna1-$rna0)/($cod1-$cod0); 
	$a = ($cod1*$rna0-$cod0*$rna1)/($cod1-$cod0); 
	$xa =($y1-$a)/$m;
	$xb =($y2-$a)/$m;
	system ("$scripts/plotxy.pl  $m $a  $xa $xb  > $output/extraline\n");
    }
    else 
    {
	system ("$scripts/plotxconst.pl  $cod0 $y1 $y2 > $output/extraline\n");
    }
}
else
{
system ("$scripts/plotxconst.pl  0 $y1 0 > $output/extraline\n");
}

my $intersect = (1.-1.4142)*$cutoff;

system ("$scripts/plotxconst.pl  0 $y1 0 > $output/x0\n");

system ("$scripts/plotxconst.pl  $cutoff $y1  $intersect > $output/plxp\n");
system ("$scripts/plotxconst.pl -$cutoff $y1 -$cutoff    > $output/plxm\n");

system ("$scripts/plotyconst.pl  $cutoff $x1  $intersect > $output/plyp\n");
system ("$scripts/plotyconst.pl -$cutoff $x1 -$cutoff    > $output/plym\n");

system ("$scripts/plotxy.pl  1  $cutoff2 $intersect $x2 > $output/plxyp\n");
system ("$scripts/plotxy.pl  1 -$cutoff2 $cutoff    $x2 > $output/plxym\n");

$num = 0;
$separate  = 1;
$identical = 1;
while($num < $num_files) {
    system ("$scripts/plot.pl $file[$num] $case $logodds $separate $identical > $file[$num]$num.plot\n");
    system ("$scripts/plot.pl $file[$num] $case  $logodds 1 0 > $output/foo1.plot\n");
    $num++;
}
$num = 0;

open(GP,'|'.GNUPLOT) || die "Gnuplot: $!";

print GP "set terminal postscript default color solid\n";
print GP "set output '$file[0].ps'\n";
print GP "set nozeroaxis\n";
print GP "set nokey\n";
print GP "set label 'RNA' at -80,50\n";
print GP "set label 'COD' at 20,-20\n";
print GP "set label 'IND' at -50,-20\n";
print GP "modx(x) = (x>0)? x:0\n";

if ($logodds =~ "LOD") {
    print GP "set title 'Posterior LODS  $tag'\n"; 
    print GP "set xlabel 'COD log-odds'\n";
    print GP "set ylabel 'RNA log-odds'\n";
} elsif  ($logodds =~ "SC") {
    print GP "set title 'SCORES.   $tag'\n"; 
    print GP "set xlabel 'COD score'\n";
    print GP "set ylabel 'RNA score'\n";
}
print GP "plot [$x1:$x2] [$y1:$y2] modx(x) with lines 18, '$output/extraline' with lines 8, '$output/x0' with lines 18,'$output/plxp' with lines 0, '$output/plxm' with lines 0, '$output/plyp' with lines 0, '$output/plym' with lines 0, '$output/plxyp' with lines 0, '$output/plxym' with lines 0, '$output/foo1.plot' using 3:4 3, '$file[0]0.plot' using 3:4 1\n";

close GP;

$num = 0;
while($num < $num_files) 
{
    system ("rm $file[$num]$num.plot\n");
    $num++;
}
system ("rm $output/extraline\n");
system ("rm $output/x0\n");
system ("rm $output/plxp\n");
system ("rm $output/plxm\n");
system ("rm $output/plyp\n");
system ("rm $output/plym\n");
system ("rm $output/plxyp\n");
system ("rm $output/plxym\n");

system ("ghostview -landscape -magstep -2 $file[0].ps&\n");





