#!/bin/perl

#phase_count.pl

use strict;

use constant GNUPLOT => '/usr/bin/gnuplot';
#use constant GNUPLOT => '/usr/freeware/bin/gnuplot';

# Print a helpful message if the user provides no input file.
if (!@ARGV) { 
        print "usage:   phase_count.pl file org1 org2 case[0-6] cutoff [x1] [x2] [y1] [y2] verbose \n";
        print "cases:   0=GLOBAL\n";
        print "         1=LOCAL_DIAG_VITERBI 2=LOCAL_DIAG_FORWARD\n";
        print "         3=LOCAL_SEMI_VITERBI 4=LOCAL_SEMI_FORWARD\n\n";
        print "         5=LOCAL_FULL_VITERBI 6=LOCAL_FULL_FORWARD\n\n";
        print "verbose: [OTH | COD | RNA | RNA/OTH | RNA/COD | RNA/OTH | COD/OTH | RNA/COD/OTH]\n\n";
        exit;
}
my $verbose;
undef $verbose;

my $file    = shift;
my $org1    = shift;
my $org2    = shift;
my $case    = shift;
my $cutoff  = shift;

my $id_max  = shift;
my $tag;
my $type;

#INPUT coordinates [x1,x2] [y1,y2]
my $x1 = shift;
my $x2 = shift;
my $y1 = shift;
my $y2 = shift;

#if not NULL, print hits of $verbose type
if (!@ARGV) { $verbose = 'NULL';}
else        { $verbose = shift; }

my $dir;
my $filename;

if ($file =~ /^(\S+)\/([^\/]+)$/) {
    $dir  = $1;
    $filename = $2;
}
else {
    $dir  = "";
    $filename = $file;
}
print "file: $filename\n";
print "dir:  $dir/\n";

my $output   = "$dir";
my $qrna_scripts  = "/nfs/wol2/people/elena/qrna/scripts";
my $outplot  = "foo.plot";
my $outfile  = "$file.stat";
system("rm $outfile\n");

if   ($case==0) { $tag = "GLOBAL";        }
elsif($case==1) { $tag = "LOCAL_DIAG_VITERBI"; }
elsif($case==2) { $tag = "LOCAL_DIAG_FORWARD"; }
elsif($case==3) { $tag = "LOCAL_SEMI_VITERBI"; }
elsif($case==4) { $tag = "LOCAL_SEMI_FORWARD"; }
elsif($case==5) { $tag = "LOCAL_FULL_VITERBI"; }
elsif($case==6) { $tag = "LOCAL_FULL_FORWARD"; }

my $overlap = 4; #minimum overlap required when calculating loci

my $sqrt2 = sqrt(2.0);

my $num = 0;

my $annotation = "/nfs/wol2/people/elena/ecoli/m52orfs.txt";
my %b;
my %c;
my $lend;
my $rend;

my @lend;
my %rend;
my %genename;
my %fctgroup;

my $idx;

my %hasblast;
my %iscod;
my %isrna;

#
#
# GO through the annotation file
#
$idx = 0;
open (ANNOTE,"$annotation") || die;
while (<ANNOTE>) {
    if (/^(\S+)\s+(CDS)\s+(\S+)\s+(\S+)\s+(\S+)\s+.+\d+\s+(\D+)s*$/) { 
	$lend[$idx]     = $4;
	$b{$lend[$idx]} = $1;
	$c{$lend[$idx]} = 0;

	$rend{$lend[$idx]}     = $5;
	$genename{$lend[$idx]} = $2."_".$3; 
	$fctgroup{$lend[$idx]} = $6;
  
	$idx++;
    }
    elsif (/^(\S+)\s+(.+RNA)\s+(\S+)\s+(\S+)\s+(\S+)/) { 
	$lend[$idx]     = $4;
	$b{$lend[$idx]} = $1;

	$rend{$lend[$idx]}     = $5;
	$genename{$lend[$idx]} = $2."_".$3; 
	$fctgroup{$lend[$idx]} = "\n";
	
	$idx++;
    }
}
close (ANNOTE);

#
#
# some info about the annotation
#
my $ann;
my $type;
my $no_tot = 0;
my $cds_tot = 0;
my $cds_putative   = 0;
my $cds_functional = 0;

if (%fctgroup) {
    foreach my $r (keys (%fctgroup)) {

	$b{$r} =~ s/b/B/;

	$ann  = $fctgroup{$r};
	$type = $genename{$r};
	
	$hasblast{$r} =  0;
	$iscod{$r}    = 0;
	$isrna{$r}    = 0;

	$no_tot ++;
	
	if ($type =~ /CDS/) {
	    $cds_tot ++;
	    if ($ann =~ /Putative/ || 
		$ann =~ /Hypothetical/ || 
		$ann =~ /unclassified/ || 
		$ann =~ /unknown/) 
	    {
		$cds_putative ++; 
	    }
	    else 
	    { 
		$cds_functional ++; 
	    }
	}
	
    }
}
print "ECOLI annotation\n";
print "#CDS functional   $cds_functional / $cds_tot\n";
print "#CDS unknown      $cds_putative   / $cds_tot\n";
print "#RNA            ",$no_tot-$cds_tot. "  / $no_tot\n\n";


my $coor1;
my $coor2;
my $coor1_new;
my $coor2_new;

my $oth;
my $rna;
my $cod;

my $name1;
my $name2;
my $name1_new;
my $name2_new;

my @name1;
my @name2;

my @lloci1;
my @lloci2;

my @rloci1;
my @rloci2;

my @type1;
my @type2;

my @howmany1;
my @howmany2;

my @codsc1;
my @codsc2;

my @rnasc1;
my @rnasc2;

my $nnamesseq = 0;   # total number of blast hits 
my $ntseq = 0;       # total number of windows
my $nloci1 = 0;      # total number of independent loci
my $nloci2 = 0;      # total number of independent loci

my $nrcseq  = 0; # number of seq in transition RNA/COD
my $nroseq  = 0; # number of seq in transition RNA/OTH
my $ncoseq  = 0; # number of seq in transition COD/OTH
my $n3seq   = 0; # number of seq in transition RNA/COD/OTH
my $nundseq;     # number of seqs in transitions
                 # $nundseq = nrcseq + $nroseq + $ncoseq + $n3seq

my $nrseq     = 0; # number of seq in RNA phase
my $ncseq     = 0; # number of seq in COD phase
my $noseq     = 0; # number of seq in OTH phase
my $nphaseseq;     # number of seq that belong to full phases
                   # $nphaseseq = $nundseq + $nrseq + $ncseq + $noseq

my $codpos = 0; # number of windows with cod_lodsc > 0
my $rnapos = 0; # number of windows with rna_lodsc > 0

my $codcut = 0; # number of windows with cod_lodsc > cutoff
my $rnacut = 0; # number of windows with rna_lodsc > cutoff

my $seq = 0;

my $realtype; 

my $idx = 0;
my $yes= 0;

my $len;
my $len_new;
my $id;
my $id_new;

my $nblastid=0;

open (FILE,"$file") || die;
while (<FILE>) {

    if (/^posX: (.+)$/ ) { 
	$coor1_new = $1;
    } 
    
    elsif (/^posY: (.+)$/) { 
	$coor2_new = $1;
	
	if ($ntseq > 0 && $id <= $id_max) {
	    #
	    #Identify type of hit, and add them up to generate some statistics at the end.
	    #
	    $realtype = decide_type($cutoff, $cod, $rna);
	    
	    $yes = 1;
	}
	
	if ($nnamesseq > $idx) 
	{ 
	    $name1 = $name1_new; 
	    $name2 = $name2_new; 
	    
	    $len = $len_new; 
	    $id  = $id_new; 
	    if ($id <= $id_max) { $ntseq++; }
	    
    #$idx = $nnamesseq; 
	}
	
	$coor1 = $coor1_new;
	$coor2 = $coor2_new;
	
    } 
    
    elsif (/^length alignment:\s+(\S+) \(id=(\d+\.\d+)\)/) { 
	$len_new = $1;
	$id_new  = $2;
    }
    
    elsif (/^>(\S+)/ && $seq == 0) { 
	$name1_new = "$1";

	$seq = 1; 
	$nnamesseq++; 

	if (%fctgroup) { 
	    foreach my $r (keys (%fctgroup)) { 
		
		if ($name1_new =~ $b{$r}) { $hasblast{$r} =  1; }
	    } 
	}    
    }
    
    elsif (/^>(\S+)/ && $seq == 1) { 
	$name2_new = "$1";

	if ($nnamesseq > 0 && $yes == 1) {
	    $nblastid++;
	}

	$yes = 0;
	$seq = 0; 
	$nnamesseq++; 
    } 
    
    elsif (/^$tag/) { $num = 1; }

    elsif (/winner = (\S+)/) { 
	$type = $1; 

	if (%fctgroup) { 
	    foreach my $r (keys (%fctgroup)) { 
		if ($name1 =~ $b{$r}) {
		    if ($type =~ /^COD$/) { $iscod{$r} =  1; }
		    if ($type =~ /^RNA$/) { $isrna{$r} =  1; }
		}
	    } 
	}    
    }
    
    elsif (/^\s+ OTH = \s+(\S+)/ && $num == 1) { 
	$oth = $1; 
    }

    elsif (/^\s+logoddspostOTH =\s+0.000\s+logoddspostCOD =\s+(\S+)\s+logoddspostRNA =\s+(\S+)\s+$/
	   && $num == 1) { 
	$cod = $1; 
	$rna = $2; 
	$num = 0;
    }
    
    else  { next; }

}

close (FILE);

$realtype = decide_type($cutoff, $cod, $rna);

print "\n---------------Some General Statistics-------------------\n";
print "FILE:            \t", "$file\n";
print "method:          \t", "$tag\n";
print "Cutoff:          \t", "$cutoff\n\n";
print "# blastn hits:   \t",  $nnamesseq/2.0 ,"\n";
print "# blastn id:    \t",  $nblastid ,"\n";
print "# windows:       \t", "$ntseq\n";
print "---------------------------------------------------------\n";

print "\n---------------Statistics by Windows---------------------\n";
print "# windows:            \t", "$ntseq\n";
print "\nRNA>0:           \t", $rnapos,    "/", "$ntseq\n";
print "\RNA>cutoff:       \t", $rnacut,    "/", "$ntseq\n\n";

print "\COD>0:            \t", $codpos,    "/", "$ntseq\n";
print "\COD>cutoff:       \t", $codcut,    "/", "$ntseq\n\n";

$nphaseseq = $nrseq + $ncseq + $noseq;
print " in phases:        \t", $nphaseseq, "/", "$ntseq\n";
print "\tRNA:             \t", $nrseq,     "/", "$nphaseseq\n";
print "\tCOD:             \t", $ncseq,     "/", "$nphaseseq\n";
print "\tOTH:             \t", $noseq,     "/", "$nphaseseq\n\n";

$nundseq   = $nrcseq + $nroseq + $ncoseq + $n3seq;
print " in transitions:   \t", $nundseq, "/", "$ntseq\n";
print "\tRNA/COD:         \t", $nrcseq,  "/", "$nundseq\n";
print "\tRNA/OTH:         \t", $nroseq,  "/", "$nundseq\n";
print "\tCOD/OTH:         \t", $ncoseq,  "/", "$nundseq\n";
print "\tRNA/COD/OTH:     \t", $n3seq,   "/", "$nundseq\n";
print "---------------------------------------------------------\n";



my $nblast = 0;
my $ncod = 0;
my $nrna = 0;
my $ncodex = 0;
my $nrnaex = 0;
my $ncodrna = 0;

if (%fctgroup) { 
    foreach my $r (keys (%fctgroup)) 
    { 
	$nblast += $hasblast{$r}; 
	$ncod += $iscod{$r};
	$nrna += $isrna{$r};

	if ($iscod{$r} > 0  && $isrna{$r} == 0) { $ncodex ++; }
	if ($iscod{$r} == 0 && $isrna{$r} > 0)  { $nrnaex ++; }
	if ($iscod{$r} >  0 && $isrna{$r} > 0)  { $ncodrna ++; }
    } 
}

  print " inblast: \t $nblast\n";
  print " cod:     \t $ncod\n";
  print " rna:     \t $nrna\n";

  print " cod_ex:  \t $ncodex\n";
  print " rna_ex:  \t $nrnaex\n";
  print " both:    \t $ncodrna\n";
  

#######################################




sub by_mostly_numeric {
    ($a <=> $b) || ($a cmp $b);
}


sub decide_type {
    my ($cutoff, $codsc, $rnasc) = @_;
    my $type;

    if ($codsc > 0.0) { $codpos++; }
    if ($rnasc > 0.0) { $rnapos++; }
    if ($codsc > $cutoff) { $codcut++; }
    if ($rnasc > $cutoff) { $rnacut++; }
    
    if ($codsc <= $cutoff && $codsc >= -$cutoff &&
	$rnasc <= $cutoff && $rnasc >= -$cutoff) { 
	$n3seq++; $type = 'RNA/COD/OTH'; 
    }
    elsif ($codsc < -$cutoff &&
	   $rnasc <= $cutoff && $rnasc >= -$cutoff) { 
	$nroseq++; $type = 'RNA/OTH'; 
    }
    elsif ($rnasc < -$cutoff &&
	   $codsc <= $cutoff && $codsc >= -$cutoff) { 
	$ncoseq++; $type = 'COD/OTH'; 
    }
    elsif ($rnasc <= $codsc+$sqrt2*$cutoff && $rnasc >= $codsc-$sqrt2*$cutoff) { 
	$nrcseq++; $type = 'RNA/COD'; 
    }
    elsif ($rnasc < -$cutoff && $codsc < -$cutoff) { 
	$noseq++; $type = 'OTH'; 
    } 
    elsif ($codsc > $cutoff && $rnasc < $codsc-$sqrt2*$cutoff) { 
	$ncseq++; $type = 'COD'; 
    } 
    elsif ($rnasc > $cutoff && $codsc < $rnasc-$sqrt2*$cutoff) { 
	$nrseq++; $type = 'RNA'; 
    } 
    else { print "unknown type for (rnasc, codsc) = (", $rnasc, ", $codsc)\n"; }

    return $type;
}



