#!/bin/perl

#phase_count.pl

use strict;

use constant GNUPLOT => '/usr/bin/gnuplot';
#use constant GNUPLOT => '/usr/freeware/bin/gnuplot';

# Print a helpful message if the user provides no input file.
if (!@ARGV) { 
        print "usage:         phase_count_fast.pl file org1 org2 case[0-6] cutoff id_max whichorg[1|2] typetarget \n";
        print "cases:         0=GLOBAL\n";
        print "               1=LOCAL_DIAG_VITERBI 2=LOCAL_DIAG_FORWARD\n";
        print "               3=LOCAL_SEMI_VITERBI 4=LOCAL_SEMI_FORWARD\n\n";
        print "               5=LOCAL_FULL_VITERBI 6=LOCAL_FULL_FORWARD\n\n";
        print "typetarget:    [OTH | COD | RNA | RNA/OTH | RNA/COD | RNA/OTH | COD/OTH | RNA/COD/OTH]\n\n";
        exit;
}
my $verbose;
undef $verbose;

my $file    = shift;
my $org1    = shift;
my $org2    = shift;
my $case    = shift;
my $cutoff  = shift;
my $id_max  = shift;
my $tag;
my $type;

#INPUT coordinates [x1,x2] [y1,y2]
my $org = shift;
if ($org != 1 && $org != 2) { print "organism has to be '1' or '2'\n"; die }

my $typetarget = shift;
my $verbose = $typetarget;

my $dir;
my $filename;

if ($file =~ /^(\S+)\/([^\/]+)$/) {
    $dir  = $1;
    $filename = $2;
}
else {
    $dir  = "";
    $filename = $file;
}
print "file: $filename\n";
print "dir:  $dir/\n";

my $output   = "$dir";
my $qrna_scripts  = "/nfs/wol2/people/elena/qrna/scripts";
my $outplot  = "foo.plot";

if   ($case==0) { $tag = "GLOBAL";        }
elsif($case==1) { $tag = "LOCAL_DIAG_VITERBI"; }
elsif($case==2) { $tag = "LOCAL_DIAG_FORWARD"; }
elsif($case==3) { $tag = "LOCAL_SEMI_VITERBI"; }
elsif($case==4) { $tag = "LOCAL_SEMI_FORWARD"; }
elsif($case==5) { $tag = "LOCAL_FULL_VITERBI"; }
elsif($case==6) { $tag = "LOCAL_FULL_FORWARD"; }

my $overlap = 4; #minimum overlap required when calculating loci

my $sqrt2 = sqrt(2.0);

my $num = 0;

my $coor1;
my $coor2;
my $coor1_new;
my $coor2_new;

my $oth;
my $rna;
my $cod;

my $name1;
my $name2;
my $name1_new;
my $name2_new;

my @name1;
my @name2;

my @lloci1;
my @lloci2;

my @rloci1;
my @rloci2;

my @type1;
my @type2;

my @howmany1;
my @howmany2;

my @codsc1;
my @codsc2;

my @rnasc1;
my @rnasc2;

my $startblast1;
my $startblast2;
my $startblast1_new;
my $startblast2_new;

my $nnamesseq = 0;   # total number of blast hits 
my $ntseq = 0;       # total number of windows
my $nloci1 = 0;      # total number of independent loci
my $nloci2 = 0;      # total number of independent loci

my $nrcseq  = 0; # number of seq in transition RNA/COD
my $nroseq  = 0; # number of seq in transition RNA/OTH
my $ncoseq  = 0; # number of seq in transition COD/OTH
my $n3seq   = 0; # number of seq in transition RNA/COD/OTH
my $nundseq;     # number of seqs in transitions
                 # $nundseq = nrcseq + $nroseq + $ncoseq + $n3seq

my $nrseq     = 0; # number of seq in RNA phase
my $ncseq     = 0; # number of seq in COD phase
my $noseq     = 0; # number of seq in OTH phase
my $nphaseseq;     # number of seq that belong to full phases
                   # $nphaseseq = $nundseq + $nrseq + $ncseq + $noseq

my $codpos = 0; # number of windows with cod_lodsc > 0
my $rnapos = 0; # number of windows with rna_lodsc > 0

my $codcut = 0; # number of windows with cod_lodsc > cutoff
my $rnacut = 0; # number of windows with rna_lodsc > cutoff

my $seq = 0;

my $realtype; 

my $idx = 0;

my $len;
my $len_new;
my $id;
my $id_new;

open (FILE,"$file") || die;
while (<FILE>) {

    if (/^posX: (.+)$/ ) { 
	$coor1_new = $1;
    } 
    
    elsif (/^posY: (.+)$/) { 
	$coor2_new = $1;
	
	if ($ntseq > 0 && $id <= $id_max) {
	    #
	    #Identify type of hit, and add them up to generate some statistics at the end.
	    #
	    $realtype = decide_type($cutoff, $cod, $rna);
	    
	    #
	    #Identify independent loci for the window just before.
	    #
	    $nloci1 = identify_loci($ntseq, $overlap, $startblast1, $name1, $coor1, $realtype, 
				    \$nloci1, \@name1, \@lloci1, \@rloci1, \@type1, \@howmany1, \@codsc1, \@rnasc1);
	    $nloci2 = identify_loci($ntseq, $overlap, $startblast2, $name2, $coor2, $realtype,
				    \$nloci2, \@name2, \@lloci2, \@rloci2, \@type2, \@howmany2, \@codsc2, \@rnasc2);
	}
	
	if ($nnamesseq > $idx) 
	{ 
	    
	    $name1 = $name1_new; 
	    $name2 = $name2_new; 
	    
	    $len = $len_new; 
	    $id  = $id_new; 
	    if ($id <= $id_max) { $ntseq++; }
	    
	    $startblast1 = $startblast1_new;
	    $startblast2 = $startblast2_new;
	    
	    #$idx = $nnamesseq; 
	}
	
	$coor1 = $coor1_new;
	$coor2 = $coor2_new;
	
    } 
    
    elsif (/^length alignment:\s+(\S+) \(id=(\d+\.\d+)\)/) { 
	$len_new = $1;
	$id_new  = $2;
    }
    
    elsif (/^>(\S+)-(\d+[><]\d+)-\s+\(\d+\)/ && $seq == 0) { 
	$name1_new = "$1";
	$startblast1_new = $2; 

	$seq = 1; 
	$nnamesseq++; 
    }
    
    elsif (/^>(\S+)-(\d+[><]\d+)-\s+\(\d+\)/ && $seq == 1) { 
	$name2_new = "$1";
	$startblast2_new = $2;

	$seq = 0; 
	$nnamesseq++; 
    } 
    
    elsif (/^$tag/) { $num = 1; }

    elsif (/winner = (\S+)/) { $type = $1; }

    elsif (/^\s+ OTH = \s+(\S+)/ && $num == 1) { $oth = $1; }

    elsif (/^\s+logoddspostOTH =\s+0.000\s+logoddspostCOD =\s+(\S+)\s+logoddspostRNA =\s+(\S+)\s+$/
	   && $num == 1) 
    { $cod = $1; $rna = $2; $num = 0;}
    
    else  { next; }

}

close (FILE);

$realtype = decide_type($cutoff, $cod, $rna);
   
$nloci1 = identify_loci($ntseq, $overlap, $startblast1, $name1, $coor1, $realtype,
			\$nloci1, \@name1, \@lloci1, \@rloci1, \@type1, \@howmany1, \@codsc1, \@rnasc1);
$nloci2 = identify_loci($ntseq, $overlap, $startblast2, $name2, $coor2, $realtype,
			\$nloci2, \@name2, \@lloci2, \@rloci2, \@type2, \@howmany2, \@codsc2, \@rnasc2);


print "\n---------------Some General Statistics-------------------\n";
print "FILE:            \t", "$file\n";
print "method:          \t", "$tag\n";
print "Cutoff:          \t", "$cutoff\n\n";
print "max id:          \t", "$id_max\n\n";
print "# blastn hits:   \t",  $nnamesseq/2.0 ,"\n";
print "# windows:       \t", "$ntseq\n";
print "---------------------------------------------------------\n";

print "\n---------------Statistics by Windows---------------------\n";
print "# windows:            \t", "$ntseq\n";
print "\nRNA>0:           \t", $rnapos,    "/", "$ntseq\n";
print "\RNA>cutoff:       \t", $rnacut,    "/", "$ntseq\n\n";

print "\COD>0:            \t", $codpos,    "/", "$ntseq\n";
print "\COD>cutoff:       \t", $codcut,    "/", "$ntseq\n\n";

$nphaseseq = $nrseq + $ncseq + $noseq;
print " in phases:        \t", $nphaseseq, "/", "$ntseq\n";
print "\tRNA:             \t", $nrseq,     "/", "$nphaseseq\n";
print "\tCOD:             \t", $ncseq,     "/", "$nphaseseq\n";
print "\tOTH:             \t", $noseq,     "/", "$nphaseseq\n\n";

$nundseq   = $nrcseq + $nroseq + $ncoseq + $n3seq;
print " in transitions:   \t", $nundseq, "/", "$ntseq\n";
print "\tRNA/COD:         \t", $nrcseq,  "/", "$nundseq\n";
print "\tRNA/OTH:         \t", $nroseq,  "/", "$nundseq\n";
print "\tCOD/OTH:         \t", $ncoseq,  "/", "$nundseq\n";
print "\tRNA/COD/OTH:     \t", $n3seq,   "/", "$nundseq\n";
print "---------------------------------------------------------\n";

if ($org == 1) {
#
#for QUERY
#
    arrange_loci($org1, $nloci1, \@name1, \@lloci1, \@rloci1, \@type1, \@howmany1, \@codsc1, \@rnasc1);
    $nloci1 = recheck_loci(\$nloci1, \@name1, \@lloci1, \@rloci1, \@type1, \@howmany1, \@codsc1, \@rnasc1);
    
    loci_stat($org1, $nloci1, \@type1);
    write_output($nloci1, \@name1, \@lloci1, \@rloci1, \@type1, \@howmany1, \@codsc1, \@rnasc1);
    class_stat($verbose, $org1, $nloci1, \@name1, \@lloci1, \@rloci1, \@type1, \@howmany1, \@codsc1, \@rnasc1);
    
}
else {
#for SUBJ
#
    arrange_loci($org2, $nloci2, \@name2, \@lloci2, \@rloci2, \@type2, \@howmany2, \@codsc2, \@rnasc2);
    $nloci2 = recheck_loci(\$nloci2, \@name2, \@lloci2, \@rloci2, \@type2, \@howmany2, \@codsc2, \@rnasc2);
    
    loci_stat($org2, $nloci2, \@type2);
    write_output($nloci2, \@name2, \@lloci2, \@rloci2, \@type2, \@howmany2, \@codsc2, \@rnasc2);
    class_stat($verbose, $org2, $nloci2, \@name2, \@lloci2, \@rloci2, \@type2, \@howmany2, \@codsc2, \@rnasc2);
}



#######################################



sub arrange_loci {
    my ($org, $nloci, $name_ref, $lloci_ref, $rloci_ref, $type_ref, $howmany_ref, $codsc_ref, $rnasc_ref) = @_;
    my $l;
    my $n;
    my $k;
    my $lend;
    my $lstart = 0;
    my $lstart_n;
    my $cur_name;
    my @list;
    my @newlist;
    my %rend;
    my %type;
    my %howmany;
    my %codsc;
    my %rnasc;
    my $howmany_tot_bf = 0;
    my $howmany_tot_af = 0;
    my $count = 0;

    $cur_name = $name_ref->[0];

    for ($l = 0; $l < $nloci; $l++) { $howmany_tot_bf += $howmany_ref->[$l]; }
#    if ($howmany_tot_bf != $ntseq) { die "bad identification of loci in $org"; }
    
    for ($l = 1; $l < $nloci; $l++) {
   	if ($cur_name =~ /^$name_ref->[$l]$/) {
	    $n++;
	}
	else { $cur_name = $name_ref->[$l]; $n = 0; $lstart_n = $l; }

	if ($n == 0) { 
	    undef @list; 
	    undef @newlist;
	    undef %rend; 
	    undef %type; 
	    undef %howmany; 
	    undef %codsc; 
	    undef %rnasc; 
	    for ($k = 0; $k < $lstart_n - $lstart; $k++) { 
		$list[$k] = $lloci_ref->[$k+$lstart]."-".$rloci_ref->[$k+$lstart].$type_ref->[$k+$lstart]; 
		$lend = $list[$k];

		$rend{$lend}    = $rloci_ref->[$k+$lstart];
		$type{$lend}    = $type_ref->[$k+$lstart];
		$howmany{$lend} = $howmany_ref->[$k+$lstart];
		$codsc{$lend}   = $codsc_ref->[$k+$lstart];
		$rnasc{$lend}   = $rnasc_ref->[$k+$lstart];
	    }
	    
	    @newlist = sort by_mostly_numeric @list;

	    for ($k = 0; $k < $lstart_n - $lstart; $k++) {
		$lend = $newlist[$k];
		if ($lend) {
		    $count++;
		    if ($lend =~ /^(\d+)-/) { $lloci_ref->[$k+$lstart] = $1; }
		    $rloci_ref->[$k+$lstart]   = $rend{$lend};
		    $type_ref->[$k+$lstart]    = $type{$lend};
		    $howmany_ref->[$k+$lstart] = $howmany{$lend};
		    $codsc_ref->[$k+$lstart]   = $codsc{$lend};
		    $rnasc_ref->[$k+$lstart]   = $rnasc{$lend};
		}
	    }
	    $lstart = $lstart_n;
	}
    }
  
#last case
    $lstart_n = $lstart + $n + 1;
    undef @list;
    undef @newlist;
    undef %rend;
    undef %type;
    undef %howmany;
    undef %codsc;
    undef %rnasc;
    
   for ($k = 0; $k < $lstart_n - $lstart; $k++) {
 	$lend = $lloci_ref->[$k+$lstart]."-".$rloci_ref->[$k+$lstart].$type_ref->[$k+$lstart];
	
	$list[$k] = $lend;
	$rend{$lend}    = $rloci_ref->[$k+$lstart];
	$type{$lend}    = $type_ref->[$k+$lstart];
	$howmany{$lend} = $howmany_ref->[$k+$lstart];
	$codsc{$lend}   = $codsc_ref->[$k+$lstart];
	$rnasc{$lend}   = $rnasc_ref->[$k+$lstart];
    }
    
    @newlist = sort by_mostly_numeric  @list;

    for ($k = 0; $k < $lstart_n - $lstart; $k++) {
	$lend = $newlist[$k];
	
	if ($lend) {
	    $count++;
	    
	    if ($lend =~ /^(\d+)-/) { $lloci_ref->[$k+$lstart] = $1; }
	    $rloci_ref->[$k+$lstart]   = $rend{$lend};
	    $type_ref->[$k+$lstart]    = $type{$lend};
	    $howmany_ref->[$k+$lstart] = $howmany{$lend};
	    $codsc_ref->[$k+$lstart]   = $codsc{$lend};
	    $rnasc_ref->[$k+$lstart]   = $rnasc{$lend};

	}
    }
    
    for ($l = 0; $l < $nloci; $l++) { $howmany_tot_af += $howmany_ref->[$l]; }
    
    #paranoia
    if ($count != $nloci) { die "bad loci count in $org: $count $nloci"; }
    if ($howmany_tot_bf != $howmany_tot_af) { die "bad blasthit count in $org: $howmany_tot_bf $howmany_tot_af"; }
    
}

sub by_mostly_numeric {
    ($a <=> $b) || ($a cmp $b);
}


sub decide_type {
    my ($cutoff, $codsc, $rnasc) = @_;
    my $type;

    if ($codsc > 0.0) { $codpos++; }
    if ($rnasc > 0.0) { $rnapos++; }
    if ($codsc > $cutoff) { $codcut++; }
    if ($rnasc > $cutoff) { $rnacut++; }
    
    if ($codsc <= $cutoff && $codsc >= -$cutoff &&
	$rnasc <= $cutoff && $rnasc >= -$cutoff) { 
	$n3seq++; $type = 'RNA/COD/OTH'; 
    }
    elsif ($codsc < -$cutoff &&
	   $rnasc <= $cutoff && $rnasc >= -$cutoff) { 
	$nroseq++; $type = 'RNA/OTH'; 
    }
    elsif ($rnasc < -$cutoff &&
	   $codsc <= $cutoff && $codsc >= -$cutoff) { 
	$ncoseq++; $type = 'COD/OTH'; 
    }
    elsif ($rnasc <= $codsc+$sqrt2*$cutoff && $rnasc >= $codsc-$sqrt2*$cutoff) { 
	$nrcseq++; $type = 'RNA/COD'; 
    }
    elsif ($rnasc < -$cutoff && $codsc < -$cutoff) { 
	$noseq++; $type = 'OTH'; 
    } 
    elsif ($codsc > $cutoff && $rnasc < $codsc-$sqrt2*$cutoff) { 
	$ncseq++; $type = 'COD'; 
    } 
    elsif ($rnasc > $cutoff && $codsc < $rnasc-$sqrt2*$cutoff) { 
	$nrseq++; $type = 'RNA'; 
    } 
    else { print "unknown type for (rnasc, codsc) = (", $rnasc, ", $codsc)\n"; }

    return $type;
}

sub decide_type_loci {
    my ($cutoff, $codsc, $rnasc) = @_;
    my $type;
    my $sqrt2 = sqrt(2.0);

    if ($codsc <= $cutoff && $codsc >= -$cutoff &&
	$rnasc <= $cutoff && $rnasc >= -$cutoff) { 
	$type = 'RNA/COD/OTH'; 
    }
    elsif ($codsc < -$cutoff &&
	   $rnasc <= $cutoff && $rnasc >= -$cutoff) { 
	$type = 'RNA/OTH'; 
    }
    elsif ($rnasc < -$cutoff &&
	   $codsc <= $cutoff && $codsc >= -$cutoff) { 
	$type = 'COD/OTH'; 
    }
    elsif ($rnasc <= $codsc+$sqrt2*$cutoff && $rnasc >= $codsc-$sqrt2*$cutoff) { 
	$type = 'RNA/COD'; 
    }
    elsif ($rnasc < -$cutoff && $codsc < -$cutoff) { 
	$type = 'OTH'; 
    } 
    elsif ($codsc > $cutoff && $rnasc < $codsc-$sqrt2*$cutoff) { 
	$type = 'COD'; 
    } 
    elsif ($rnasc > $cutoff && $codsc < $rnasc-$sqrt2*$cutoff) { 
	$type = 'RNA'; 
    } 
    else { print "unknown type for (rnasc, codsc) = (", $rnasc, ", $codsc)\n"; }

    return $type;
}




sub identify_loci {
    my ($ntseq, $overlap, $startblast, $name, $coor, $type, $nloci_ref, $name_ref, $lloci_ref, $rloci_ref, 
	$type_ref, $howmany_ref, $codsc_ref, $rnasc_ref) = @_;
    my $l;
    my $coor1;
    my $coor2;
    my $startwin;
    my $endwin;
    my $new = 1;
    my $nloci;
    my $abs;
   
    $nloci = $$nloci_ref;
    
    if ($coor =~ /^\d+-\d+\s+\[(\d+)-(\d+)\]\((\d+)\)/) {

	#remember conventions for qrna output:
	#
	#      posX: 0-62 [0-59](60) 
	#
	# is an alignment of 63 positions with 3 gaps. 
	# So the actual positions are from 0 to 59 not to 62.
	#
	#
	$coor1 = $1;
	$coor2 = $2;
    }

    #get the ENDS depending on the strand
    #
    if ($startblast =~ /(\d+)>(\d+)/) {
	$startwin = $1 + $coor1;
	$endwin   = $1 + $coor2;
    }
    elsif ($startblast =~ /(\d+)<(\d+)/) {
	$startwin = $2 - $coor2;
	$endwin   = $2 - $coor1;
    }
    else { print "fasta name has to indicate strand\n"; die; }

    #paranoia
    if ($startwin > $endwin) { print "got ends of the window wrong\n"; die; }
    
    if ($name =~ /(\S+)-(\d+)-(\d+)/) { 
	$name = $1; 
	if ($2 < $3) { $abs = $2; }
	else         { $abs = $3; }
	$startwin += $abs; $endwin += $abs;
    }

    $name =~ s/[\(\)]/-/g; #parentheses in the name confuse everything!
    $name =~ s/[\|]/-/g;   #vertical lines in the name confuse everything!

#start the first locus
    if ($ntseq == 1 && $type =~ /^$typetarget$/) {
	$name_ref->[$nloci]    = $name;
	$lloci_ref->[$nloci]   = $startwin;
	$rloci_ref->[$nloci]   = $endwin;
	$type_ref->[$nloci]    = $type;
	$howmany_ref->[$nloci] = 1;
	$codsc_ref->[$nloci]   = $cod;
	$rnasc_ref->[$nloci]   = $rna;
	$nloci++;
    }
    
    if ($ntseq > 1) {

        #identify if this is a new locus
	for ($l = 0; $l < $nloci; $l++) {
                #an already existing locus, modify the ends if convenient
	    if ($name     =~ /^$name_ref->[$l]$/         && 
		$type     =~ /^$type_ref->[$l]$/         &&
		$startwin <  $rloci_ref->[$l] - $overlap && 
		$endwin   >  $lloci_ref->[$l] + $overlap) 
	    {
		#modify ends
		if($startwin < $lloci_ref->[$l]) { $lloci_ref->[$l] = $startwin; }
		if($endwin   > $rloci_ref->[$l]) { $rloci_ref->[$l] = $endwin;   }
		
		$howmany_ref->[$l] ++;
		$codsc_ref->[$l]   += $cod;
		$rnasc_ref->[$l]   += $rna;
		$new = 0;
		last;
	    }
	}
	
	if ($new == 1 && $type =~ /^$typetarget$/) {
	    $name_ref->[$nloci]    = $name;
	    $lloci_ref->[$nloci]   = $startwin;
	    $rloci_ref->[$nloci]   = $endwin;
	    $type_ref->[$nloci]    = $type;
	    $howmany_ref->[$nloci] = 1;
	    $codsc_ref->[$nloci]   = $cod;
	    $rnasc_ref->[$nloci]   = $rna;

	    $nloci++;
	}
    }

    return $nloci;
}


sub loci_stat {
    my ($org, $nloci, $type_ref) = @_;
    my $l;
    my $type;
    
    my $nundloci = 0; # number of loci in transitions
    my $nrcloci  = 0; # number of loci in transition RNA/COD
    my $nroloci  = 0; # number of loci in transition RNA/OTH
    my $ncoloci  = 0; # number of loci in transition COD/OTH
    my $n3loci   = 0; # number of loci in transition RNA/COD/OTH
    
    my $nphaseloci = 0; # number of loci that belong to full phases
    my $nrloci     = 0; # number of loci in RNA phase
    my $ncloci     = 0; # number of loci in COD phase
    my $noloci     = 0; # number of loci in OTH phase
    
    for ($l = 0; $l < $nloci; $l ++) {
	$type = $type_ref->[$l];
	if    ($type =~ /^RNA$/)           { $nrloci++;  }
	elsif ($type =~ /^COD$/)           { $ncloci++;  }
	elsif ($type =~ /^OTH$/)           { $noloci++;  }
	elsif ($type =~ /^RNA\/OTH$/)      { $nroloci++; }
	elsif ($type =~ /^RNA\/COD$/)      { $nrcloci++; }
	elsif ($type =~ /^COD\/OTH$/)      { $ncoloci++; }
	elsif ($type =~ /^RNA\/COD\/OTH$/) { $n3loci++;  }
	else { die "this $org loci does not have a type, $type\n"; }
    }
    $nundloci   = $nrcloci + $nroloci + $ncoloci + $n3loci;
    $nphaseloci = $nrloci + $ncloci + $noloci;
    
    print "\n---------------Statistics by Loci ($org)------------------------\n";
    print "# loci:            \t", "$nloci\n";
    print " in phases:        \t", $nphaseloci, "/", "$nloci\n";
    print "\tRNA:             \t", $nrloci,     "/", "$nphaseloci\n";
    print "\tCOD:             \t", $ncloci,     "/", "$nphaseloci\n";
    print "\tOTH:             \t", $noloci,     "/", "$nphaseloci\n\n";
    
    print " in transitions:   \t", $nundloci, "/", "$nloci\n";
    print "\tRNA/COD:         \t", $nrcloci,  "/", "$nundloci\n";
    print "\tRNA/OTH:         \t", $nroloci,  "/", "$nundloci\n";
    print "\tCOD/OTH:         \t", $ncoloci,  "/", "$nundloci\n";
    print "\tRNA/COD/OTH:     \t", $n3loci,   "/", "$nundloci\n\n";
    print "---------------------------------------------------------\n";
}

sub write_output {
    my ($nloci, $name_ref, $lloci_ref, $rloci_ref, $type_ref, $howmany_ref, $codsc_ref, $rnasc_ref) = @_;
    my $l;

    open (OUT,">$outplot") || die;
    
    for ($l = 0; $l < $nloci; $l++) {     
	print OUT "$l-loci $name_ref->[$l] $lloci_ref->[$l] $rloci_ref->[$l] (", 
	$rloci_ref->[$l]-$lloci_ref->[$l]+1,") $howmany_ref->[$l] $type_ref->[$l] "; 
	printf OUT "%.2f %.2f\n", $codsc_ref->[$l], $rnasc_ref->[$l]; 
    }
    close (OUT);
}

sub class_stat {
    my ($verbose, $org, $nloci, $name_ref, $lloci_ref, $rloci_ref, $type_ref, $howmany_ref, $codsc_ref, $rnasc_ref) = @_;
    my $l;
    my $n = 0;
    my $ave_length = 0;
    my $nloci_type = 0;
    
    if ($verbose !~ /NULL/) {
	
	print "\n---------------Statistics for $verbose loci ($org):-------------------\n";
	
	for ($l = 0; $l < $nloci; $l++) { 
	    if($type_ref->[$l] =~ /^$verbose$/) { 
		$nloci_type ++; 
		$ave_length += $rloci_ref->[$l] - $lloci_ref->[$l]; 
	    }
	}
	

	if ($nloci_type == 0) { print "sorry no loci of type $verbose\n"; }
	else {
	    print "# loci:\t", "$nloci_type\n";
	    printf "ave_length:\t %.2f\n\n", $ave_length / $nloci_type;
	    
	    for ($l = 0; $l < $nloci; $l++) {
		if($type_ref->[$l] =~ /^$verbose$/) { 
		    $n ++; 
		    print "$n-loci $name_ref->[$l] $lloci_ref->[$l] $rloci_ref->[$l] (", 
		    $rloci_ref->[$l]-$lloci_ref->[$l]+1,") $howmany_ref->[$l] $type_ref->[$l] "; 
		    printf "%.2f %.2f\n", $codsc_ref->[$l], $rnasc_ref->[$l]; 
		}
	    }
	}
    }
}


# the reason for this subroutine is that ones we have identified the independent loci,
# we have to go back to check that those loci (which may have expanded their limits after being defined)
# do not overlap.

sub recheck_loci {
    my ($nloci_ref, $name_ref, $lloci_ref, $rloci_ref, $type_ref, $howmany_ref, $codsc_ref, $rnasc_ref) = @_;
    my $l;
    my $k;
    my $sh;
    my $name;
    my $startloci;
    my $endloci;
    my $type;
    my $howmany;
    my $codsc;
    my $rnasc;
    my $nloci;
    
    $nloci = $$nloci_ref;
    
    for ($k = 0; $k < $nloci; $k++) {
	for ($l = $k+1; $l < $nloci; $l++) {
	    $name      = $name_ref->[$l];
	    $startloci = $lloci_ref->[$l];
	    $endloci   = $rloci_ref->[$l];
	    $type      = $type_ref->[$l];
	    $howmany   = $howmany_ref->[$l];
	    $codsc     = $codsc_ref->[$l];
	    $rnasc     = $rnasc_ref->[$l];
   
	    #an already existing locus, modify the ends if convenient
	    if ($name      =~ /^$name_ref->[$k]$/        && 
		$type      =~ /^$type_ref->[$k]$/        &&
		$startloci < $rloci_ref->[$k] - $overlap && 
		$endloci   > $lloci_ref->[$k] + $overlap) 
	    {
		#modify ends
		if($startloci < $lloci_ref->[$k]) { $lloci_ref->[$k] = $startloci; }
		if($endloci   > $rloci_ref->[$k]) { $rloci_ref->[$k] = $endloci;   }
 	
		$howmany_ref->[$k] += $howmany;
		$codsc_ref->[$k]   += $codsc;
		$rnasc_ref->[$k]   += $rnasc;

		for ($sh = $l; $sh < $nloci-1; $sh++) {
		    
		    $name_ref->[$sh]    = $name_ref->[$sh+1];
		    $lloci_ref->[$sh]   = $lloci_ref->[$sh+1];
		    $rloci_ref->[$sh]   = $rloci_ref->[$sh+1];
		    $type_ref->[$sh]    = $type_ref->[$sh+1];
		    $howmany_ref->[$sh] = $howmany_ref->[$sh+1];
		    $codsc_ref->[$sh]   = $codsc_ref->[$sh+1];
		    $rnasc_ref->[$sh]   = $rnasc_ref->[$sh+1];
		}
		
		$nloci--;
		$l--;
	    }
	}
    }

    #finallly, normalize mean scores and calculate types.
    for ($l = 0; $l < $nloci; $l++) {	
	$codsc_ref->[$l] /= $howmany_ref->[$l];
	$rnasc_ref->[$l] /= $howmany_ref->[$l];

	#$type_ref->[$l] = decide_type_loci($cutoff, $codsc_ref->[$l], $rnasc_ref->[$l]);
    }
    

    return $nloci;
}

