/************************************************************
 * QRNA - Comparative analysis of biological sequences 
 *        with pair hidden Markov models and pair stochastic context-free grammars
 * Copyright (C) 2000-Howard Hughes Medical Institute
 * All Rights Reserved
 * 
 *     This source code is distributed under the terms of the
 *     GNU General Public License. See the files COPYING and LICENSE
 *     for details.
 ************************************************************/

/* funcs.h
 * ANSI prototypes for all external functions.
 * 
 */
#ifndef FUNCSH_INCLUDED
#define FUNCSH_INCLUDED

#include <stdio.h>
#include "cfg.h"
#include "globals.h"
#include "squid.h"
#include "structs.h"

/* from cfgio.c:
 */
extern void PairsSCFG (double **cfg, double **ret_pair);
extern void Pairs5SCFG (double **cfg, double **ret_pair);
extern int  ReadSCFG(FILE *ofp, double ***ret_cfg);
extern int  SaveSCFG(FILE *ofp, double **cfg);
extern void WriteRdbSCFG(FILE *ofp, double **cfg);
extern void WriteRdbLogSCFG(FILE *ofp, double **cfg);
extern void WriteRdbSummary(FILE *ofp, double **cfg);

/* from cfgmodel.c:
 */
extern double  **AllocSCFG(void);
extern double ***AllocSCFGNodes(void);
extern int     **AllocIntSCFG(void);
extern void      CheckSCFG(double **cfg);
extern void      CheckSCFGEmit(double **cfg_emit);
extern void      CheckSCFGNode(double ***cfg_node);
extern int       Connects(int dp, int node);
extern char     *dpcNAME(int node);
extern double  **DupSCFG(double **cfg);
extern void      FreeSCFG(double **cfg);
extern void      FreeSCFGNode(double ***cfg_node);
extern void      Log2ProbSCFG(double **cfg);
extern void      MdfySCFGEmit(double ***ret_cfg_emit, double *pmodel);
extern void      MdfySCFGNode(double ****ret_cfg_node);
extern int       Nemit(int node);
extern int       Ntype(int dp, int tr);
extern void      PrintPerDp(int dp, int idx, double **cfg, double **cfg_emit, double ***cfg_node);
extern void      ReassembleSCFG(double ***cfg_node, double **cfg_emit, double ***ret_cfg);
extern double    Size(int dp, int tr);
extern int       Symi(int dp, int tr);
extern int       Symj(int dp, int tr);
extern int       Symk(int dp, int tr);
extern int       Syml(int dp, int tr);
extern void      TieProbs(double **cfg, double ****ret_cfg_node, double ***ret_cfg_emit);
extern int       Ttype(int node, int symi, int symj, int symk, int syml, int size);

/* from cfgparse.c:
 */
extern void GraphicTrace(FILE *fp, struct tracekn_s *tr, char *seq, int len);
extern int  KHS2Trace(FILE *outf, char *seq, char *ss, int len, struct tracekn_s **ret_tr,
		      int allow_pseudoknots, int debug);
extern void TraceCount(char *seq, int len, double wgt, struct tracekn_s *tr, double **cfg);

/* from codingcreatealign.c:
 */
extern void SimulateCodingSequences(int L, double pammodel[64][64], char **ret_s1, char **ret_s2);
extern void SimulateCODAlign(FILE *ofp, SQINFO *sqinfoX, int *seqX, SQINFO *sqinfoY, int *seqY, 
			     int start, int L, struct codmodel_s *cod, struct nullmodel_s *null, 
			     struct ali_s *ali, int traceback, int alignment, char *string_name);
extern void SimulateCODSequence(FILE *ofp, int *seq, SQINFO *sqinfoX, int *seqX, SQINFO *sqinfoY, 
				int *seqY, int start, int *L, struct codmodel_s *cod, 
				struct nullmodel_s *null, struct ali_s *ali, int traceback, 
				int alignment, char *string_name);
/* from coddp.c:
 */
extern struct coddp_s *AllocDpDiagCOD(int L);
extern struct coddp_s *AllocDpFullCOD(int Lx, int Ly);
extern struct sc2_s   *ForwardCODDiag(FILE *ofp, int *seqX, int *seqY, int start, int L, 
				      struct codmodel_s *cod, struct dpd_s *dp, 
				      struct nullmodel_s *null, int logodds, struct dpsc3_s *sc);
extern struct sc2_s   *ViterbiCODFull(FILE *ofp, SQINFO sqinfoX, int *seqX, SQINFO sqinfoY, int *seqY,  
				      int startX, int startY, int Lx, int Ly, int *ret_leg, int Lw,
				      struct codmodel_s *cod, struct dpf_s *dp, struct nullmodel_s *null, 
				      struct ali_s *ali, int alignment, int logodds, int traceback, struct end3_s *ends);
extern void            FreeDpCOD(struct coddp_s *dp);
extern void            PatternDpDiagCOD(int L, struct coddp_s *coddp);
extern struct sc2_s   *ViterbiCODDiag(FILE *ofp, SQINFO sqinfoX, int *seqX, SQINFO sqinfoY, int *seqY, 
				      int start, int L, struct codmodel_s *cod, struct dpd_s *dp, 
				      struct nullmodel_s *null, struct ali_s *ali,
				      int alignment,  int logodds,  int traceback, struct end3_s *ends);

/* from codmodel.c:
 */
extern struct codmodel_s *AllocCODModel(void);
extern struct pamcond_s  *AllocPamCond(void);
extern void     CheckCODProbs(struct codmodel_s *cod);
extern void     CheckPAMModel(double **pammodel);
extern void     CODToLog2(struct codmodel_s *cod);
extern void     CODLog2ToOdds(struct codmodel_s *cod, struct nullmodel_s *null);
extern void     ConstructCODModel(double *mutpxy, double **pammodel, double **hexa, double tfactor, 
				  struct codmodel_s *cod, int add_hexamer, int pedantic);
extern void     ConstructCODEmitProbs(double **pammodel, double *pmut, double **pcodon);
extern void     ConstructESTModel(double **estmodel);
extern void     ConstructHexamerProbs(double **hexa, double **phexa, int add_hexamer);
extern void     ConstructPAMModel(int **pam, double scale, double tfactor, double ***ret_pammodel, 
				  int changefreq, int pedantic, int verbose);
extern void     ConstructPAMCondi(int **pam, double scale, double tfactor, struct pamcond_s **ret_pamcond, 
				  int pedantic, int verbose);
extern void     ConstructPsingle (double **pammodel, double **ret_psingle);
extern void     ConstructPAMJoint(struct pamcond_s *pamcond, double ***ret_pammodel, double *freqX, double *freqY, 
				  int pedantic, int verbose);
extern void     ConstructPAM5Model(int **pam, double scale, double ***ret_pammodel);
extern void     DefaultCodonBias(double codebias[64]);
extern void     DefaultGeneticCode(int aacode[64]);
extern void     FreeCODModel(struct codmodel_s *cod);
extern void     FreePamCond(struct pamcond_s *pamcond);
extern void     NormalizePAMModel(double **pammodel);
extern void     PatternCODModel(struct codmodel_s *cod);
extern void     PrintCodonString(int x);
extern void     PrintCODModel(struct codmodel_s *cod);
extern void     PrintPAMModel(double **pammodel, double *mutpxy);

/* from codscorealign.c:
 */
extern double        average(int pos, int x1, int x2, int codon, struct codmodel_s *cod, 
			     struct nullmodel_s *null, int logodds);
extern double        average_codon(int codon, struct codmodel_s *cod, struct nullmodel_s *null, int logodds);
extern double        fourcodonX(int xleft, int gappos, int cod5X, int cod5Y, struct codmodel_s *cod, 
				struct nullmodel_s *null, int logodds, int *ret_tr1, int *ret_tr2);
extern double        fourcodonY(int yleft, int gappos, int cod5X, int cod5Y, struct codmodel_s *cod,  
				struct nullmodel_s *null, int logodds, int *ret_tr1, int *ret_tr2);
extern int           isitagap(int x);
extern int           iscomplete(int cod5);
extern int           isgap1(int cod5);
extern int           isgap2(int cod5);
extern int           isgap3(int cod5);
extern int           isnocod(int cod5);
extern int           miss1(int cod5);
extern int           miss2(int cod5);
extern int           miss3(int cod5);
extern double        ScoreWithCOD_ungapped(int *isX, int *isY, int L, struct codmodel_s *cod, 
					   int traceback);
extern struct sc2_s *ScoreWithCOD(FILE *ofp, int *isX, int *isY, int start, int L, struct codmodel_s *cod, 
				  struct nullmodel_s *null, struct coddp_s  *coddp, int logodds, int traceback);
extern double        ScoreWithPAM(int *isX, int *isY, int L, struct codmodel_s *cod);


/* from er_math.c:
 */
extern int    DENChoose(double *p, int N);
extern int    DLog2Choose(double *p, int N);
extern double DLog2Sum(double *log2p, int n);
extern int    ILogSum(int *logp, int n);
extern int    ILog2Sum(int *log2p, int n);
extern void   FLog(float *vec, int n);
extern void   FLog2(float *vec, int n);
extern void   FExp(float *vec, int n);
extern void   FExp2(float *vec, int n);
extern void   FAddConst(float *vec, float plus, int n);
extern void   IAddConst(int *vec, int plus, int n);

/* from evolve.c
 */
extern double *Cal_Id(int L);
extern double *Cal_LUP_Decomp(FILE *ofp, double *M, int *perm, int L, int verbose);
extern double *Cal_M_Exp(FILE *ofp, double *M, int L, double power, int verbose);
extern double *Cal_M_Inv(FILE *ofp, double *M, int L, int verbose);
extern double *Cal_M_Log(FILE *ofp, double *M, int L, int verbose);
extern double *Cal_M_MinId(double *M, int L);
extern double *Cal_M_N_Prod(FILE *ofp, double *M, double *N, int Li, int Lj, int Lk, int verbose);
extern double *Cal_M_N_Sum(FILE *ofp, double *M, double *N, int Li, int Lj, int add, int verbose);
extern int     Check_Accuracy(double *vec, int L);
extern double *Condi_From_Joint(FILE *ofp, double *P, int L, int verbose);
extern double *ConditionalsEvolved(FILE *ofp, double *P, double *Q_0, int L, double tfactor, 
				   int pedantic, int verbose);
extern double *ConditionalsEvolved_2(FILE *ofp, double *Q, double *Q_0, double *R, int L, double tfactor, 
				     int pedantic, int verbose);
extern double *HessenbergForm(FILE *ofp, double *A, int L);
extern void    Hessenberg2Eigenvalues(FILE *ofp, double *H, int L, double **ret_vr, double **ret_vi);
extern void    Joint2Joint(double *p_joint, double *q_nought, int L, double tfactor, 
			   int pedantic, int verbose);
extern void    Solve_LU_Eq(FILE *ofp, double *LU, double *b, int L, int verbose);

/* from hexamer.c: Parse disk file with hexamer frequencies for diferent organisms
 */
extern int ParseHexamerFile(FILE *fp, double ***ret_hexap);

/* from misc.c: miscellaneous functions with no home
 */
extern void            AllocAli(int L, struct ali_s **ret_ali);
extern struct dpd_s   *AllocDpDiagMtx(int L);
extern struct dpf_s   *AllocDpSemiMtx(int L);
extern struct dpf_s   *AllocDpFullMtx(int Lx, int Ly);
extern struct dpsc3_s *AllocDpDiagScores(int L);
extern struct dpsc3_s *AllocDpScores(int L);
extern void            AllocModels(struct model_s **ret_model);
extern void     ConstructModels(FILE *ofp, char *hexapfile, char *pamfile, char *cfgfile, 
				struct model_s **ret_model, double tfactor, 
				int logodds, int pedantic, int verbose);
extern void     ConstructModels_phase1(FILE *ofp, char *hexapfile, char *pamfile, char *cfgfile, 
				       struct model_s **ret_model, struct pamcond_s **ret_pamcond, 
				       double ****ret_cfg_node,  
				       double ***ret_hexa, double **ret_pair5prob, 
				       int add_hexamer, int logodds, 
				       int pedantic, double tfactor, int verbose);
extern void     ConstructModels_phase2(FILE *ofp, struct pamcond_s *pamcond, double ***cfg_node, 
				       double **hexa, double *pair5prob, 
				       double *freX,  double *freqY, struct model_s *model, int add_hexamer, 
				       int changefreq, int logodds, int pedantic, int tfactor, int verbose);
extern void     ConstructTiedProbs(double **pammodel, double tfactor, double **ret_mutpxy, 
				   double **ret_mut5pxy, int pedantic, int verbose);
extern void     FormatSeqs(FILE *ofp, int Lmax, int format, char *seqX, char *seqY, 
			   SQINFO *sqinfoX, SQINFO *sqinfoY, int **ret_ctX, int **ret_ctY, 
			   int *isegX, int *isegY, int allow_pseudoknots, int verbose);
extern void     FreeAli(struct ali_s *ali);
extern void     FreeDpDiagMtx(struct dpd_s *dp);
extern void     FreeDpSemiMtx(struct dpf_s *dp);
extern void     FreeDpFullMtx(struct dpf_s *dp);
extern void     FreeDpScores(struct dpsc3_s *sc);
extern void     FreeModels(struct model_s *model);
extern void     ModelLog2ToOdds(struct model_s *model);
extern void     PatternAli(int L, struct ali_s *ali);
extern void     PatternDpDiag(int L, struct dpd_s *dp);
extern void     PatternDpFull(int Lx, int Ly, struct dpf_s *dp);
extern void     PatternModels(struct model_s *model);
extern void     PrintTrProbs(struct model_s *model);
extern void     PrintOTHTrProbs(struct othmodel_s *oth);
extern void     PrintProbs(FILE *ofp, double *prob, int L);

/* from nrnscfg.c: Dynamic programming for the Non Redundant Nussionv grammar (R Dowell) 
 */
extern struct nrn_s      *AllocNRNModel(int L);
extern struct nrnscfg_s  *AllocScfgNRN(int L);
extern void     Best_nrnS(int *sX, int *sY, struct nrn_s *nrn, int start, int L, int j, int d, 
			  struct nrnscfg_s *mtx, struct end_s *rnaends);
extern void     Best_nrnL(int *sX, int *sY, struct nrn_s *nrn, int start, int L, int j, int d, 
			  struct nrnscfg_s *mtx, struct end_s *rnaends);
extern void     Best_nrnR(int *sX, int *sY, struct nrn_s *nrn, int start, int L, int j, int d, 
			  struct nrnscfg_s *mtx, struct end_s *rnaends);
extern void     ConstructNRNModel(int L, struct nrn_s **ret_nrn, double **pp, double *ps);
extern void     Cyk_NRN(FILE *ofp, SQINFO sqinfoX, int *sX, SQINFO sqinfoY, int *sY, int start, int L,  
			struct nrn_s *nrn, struct nrnscfg_s *mtx, struct end_s *rnaends);
extern void     FreeNRNModel(struct nrn_s *nrn);
extern void     FreeScfgNRN(struct nrnscfg_s *mx);
extern void     Tr_nrnS(FILE *ofp, int *sX, int *sY, struct nrn_s *nrn, int start, int L, int j, int d, 
			struct nrnscfg_s *mtx, 
			int *flag, struct tracekn_s *curr_tr, struct traceknstack_s *dolist, int traceback, struct end_s *rnaends);
extern void     Tr_nrnL(FILE *ofp, int *sX, int *sY, struct nrn_s *nrn, int start, int L, int j, int d, 
			struct nrnscfg_s *mtx, 
			int *flag, struct tracekn_s *curr_tr, struct traceknstack_s *dolist, int traceback, struct end_s *rnaends);
extern void     Tr_nrnR(FILE *ofp, int *sX, int *sY, struct nrn_s *nrn, int start, int L, int j, int d, 
			struct nrnscfg_s *mtx, 
			int *flag, struct tracekn_s *curr_tr, struct traceknstack_s *dolist, int traceback, struct end_s *rnaends);
extern void     Trk_NRN(FILE *ofp, SQINFO sqinfoX, int *sX, SQINFO sqinfoY, int *sY, int start, int L, int j, int d,
			struct nrn_s *nrn, struct nrnscfg_s *mtx, struct tracekn_s **ret_trace, int traceback, struct end_s *rnaends);

/* from nullmodel.c:
 */
extern struct nullmodel_s  *AllocNullModel(void);
extern void    CheckNullProbs(struct nullmodel_s *null);
extern void    ConstructNullModel(double *nullpxy, struct nullmodel_s *null, struct nullparam_s param);
extern void    ConstructXemNullProb(double *nullpxy, double *xem);
extern void    ConstructYemNullProb(double *nullpxy, double *yem);
extern void    FreeNullModel(struct nullmodel_s *null);
extern double  NullAddOnePair(int cur_x, int cur_y, struct nullmodel_s *null);
extern double  NullAddOneX(int cur_x, struct nullmodel_s *null);
extern double  NullAddOneY(int cur_y, struct nullmodel_s *null);
extern void    NullToLog2(struct nullmodel_s *null);
extern void    NullLog2ToOdds(struct nullmodel_s *null1, struct nullmodel_s *null2);
extern void    PatternNullModel(struct nullmodel_s *null);
extern void    PrintNullModel(struct nullmodel_s *null);
extern double  ScoreWithNull(int *seqX, int *seqY, int startX, int Lx, int startY, int Ly, 
			     struct nullmodel_s *null);
extern double  ScoreWithNullDiag(int *sX, int *sY, int start, int L, struct nullmodel_s *null);
extern double  ScoreWithNullX(int *sX, int startX, int Lx, struct nullmodel_s *null);
extern double  ScoreWithNullY(int *sY, int startY, int Ly, struct nullmodel_s *null);
extern double  ScoreWithNullRev(int *seqX, int *seqY, int startX, int Lx, int startY, int Ly, 
				struct nullmodel_s *null);
extern double  ScoreWithNullRevDiag(int *seqX, int *seqY, int start, int L, struct nullmodel_s *null);
extern void    SimulateNullAlign(int *s1, int *s2, int L1, int L2, struct nullmodel_s *null);


/* from othercreatealign.c:
 */
extern char *SimulateNullSequence(char *s1, int L, double nullmodel[4][4]);
extern void  SimulateOTHAlign(FILE *ofp, SQINFO *sqinfoX, int *seqX, SQINFO *sqinfoY, int *seqY, 
			      int start, int L, struct othmodel_s *oth, struct ali_s *ali, 
			      int traceback, int alignment, char *string_name);
extern void  SimulateOTHSequence(FILE *ofp, int *seq, SQINFO *sqinfoX, int *seqX, SQINFO *sqinfoY, int *seqY, 
				 int start, int *L, struct othmodel_s *oth, struct ali_s *ali, 
				 int traceback, int alignment, char *string_name);
extern void SimulateMutSequence(FILE *ofp, int *seq, SQINFO *sqinfoX, int *seqX, SQINFO *sqinfoY, int *seqY, 
				int start, int L, double *pxy, struct ali_s *ali, int alignment, 
				char *string_name);
extern void SimulateMut5Sequence(FILE *ofp, int *seq, SQINFO *sqinfoX, int *seqX, SQINFO *sqinfoY, int *seqY, 
				 int start, int *L, double *pxy, struct ali_s *ali, int alignment, 
				 char *string_name);

/* from othdp.c:
 */
extern struct othdpd_s *AllocDpDiagOTH(int L);
extern struct othdpf_s *AllocDpFullOTH(int Lx, int Ly);
extern double      **AllocMatrixDiag(int L);
extern double      **AllocMatrixFull(int Lx,  int Ly);
extern void          CompareOTH(FILE *ofp, SQINFO sqinfoX, int *seqX, SQINFO sqinfoY, int *seqY, 
				int start, int L, struct othmodel_s *oth, struct othdpd_s *othdp, 
				double *sc, struct ali_s *ali, int traceback, int alignment);
extern void          CompareOTHFull(FILE *ofp, SQINFO sqinfoX, int *seqX, SQINFO sqinfoY, int *seqY, 
				    int startX, int Lx, int startY, int Ly,  int *ret_leg, int Lw,
				    struct othmodel_s *oth, struct othdpf_s *othdp, struct ali_s *ali, struct end_s *othends);
extern struct sc2_s *ForwardOTHFull(FILE *ofp, int *seqX, int *seqY, int startX, int startY, int Lx, int Ly, int Lw,
				    struct othmodel_s *oth, struct othdpf_s *othdp, double *sc);
extern double        ForwardOTH_L2(FILE *ofp, int *seqX, int *seqY, int startX, int startY, int Lx, int Ly, 
				   int Lw, struct othmodel_s *oth, struct othdpf_s *othdp, double *sc);
extern struct sc2_s *ForwardOTHDiag(FILE *ofp, int *seqX, int *seqY, int start, int L, 
				    struct othmodel_s *oth, struct othdpd_s *othdp, double *sc);
extern double        ForwardOTHDiag_L(FILE *ofp, int *seqX, int *seqY, int start, int L, 
				      struct othmodel_s *oth, struct othdpd_s *othdp, double *sc);
extern void          ForwardOTHDiagMatrix(FILE *ofp, double **mt, int *seqX, int *seqY, int start, int L, 
					  struct othmodel_s *oth, struct othdpd_s *othdp, double *sc);
extern void          ForwardOTHDiagVector(FILE *ofp, double *v, int *seqX, int *seqY, int start, int L, 
					  struct othmodel_s *oth, struct othdpd_s *othdp, double *sc);
extern void          FreeDpDiagOTH(struct othdpd_s *othdp);
extern void          FreeDpFullOTH(struct othdpf_s *othdp);
extern double        NullOJ(int iabs, int jabs, int dx, int dy, int *seqX, int *seqY, struct othmodel_s *oth);
extern void          PatternDpDiagOTH(int L, struct othdpd_s *dp);
extern void          PatternDpFullOTH(int Lx, int Ly, struct othdpf_s *dp);
extern void          PatternOTHModel(struct othmodel_s *oth);
extern void          TestForwardOTH(FILE *ofp, int *seqX, int *seqY, int start, int L, struct othmodel_s *oth, 
				    struct othdpd_s *othdp, double *sc);
extern void          TestViterbiOTH(FILE *ofp, SQINFO sqinfoX, int *seqX, SQINFO sqinfoY, int *seqY, 
				    int start, int L, 
				    struct othmodel_s *oth, struct othdpd_s *othdp, struct ali_s *ali, 
				    int alignment, int traceback, struct end_s *othends);
extern struct sc2_s *ViterbiOTHFull(FILE *ofp, 
				   SQINFO sqinfoX, int *seqX, 
				   SQINFO sqinfoY, int *seqY, 
				   int startX, int startY,
				   int Lx, int Ly, 
				   int *ret_len_ali,  int Lw,
				   struct othmodel_s *oth, struct othdpf_s *othdp, 
				   struct ali_s *ali, 
				   int alignment, int sweep, int traceback, struct end_s *othends);
extern double        ViterbiOTH_L2(FILE *ofp, 
				   SQINFO sqinfoX, int *seqX, 
				   SQINFO sqinfoY, int *seqY, 
				   int startX, int startY,
				   int Lx, int Ly, 
				   int *ret_len_ali,  int Lw,
				   struct othmodel_s *oth, struct othdpf_s *othdp, 
				   struct ali_s *ali, 
				   int alignment, int sweep, int traceback, int voutput, struct end_s *othends);
extern struct sc2_s *ViterbiOTHDiag(FILE *ofp, 
				    SQINFO sqinfoX, int *seqX, 
				    SQINFO sqinfoY, int *seqY, 
				    int start, int L, 
				    struct othmodel_s *oth, struct othdpd_s *othdp, struct ali_s *ali, 
				    int alignment, int sweep, int traceback, struct end_s *othends);
extern double        ViterbiOTHDiag_L(FILE *ofp, 
				      SQINFO sqinfoX, int *seqX, 
				      SQINFO sqinfoY, int *seqY, 
				      int start, int L, 
				      struct othmodel_s *oth, struct othdpd_s *othdp, struct ali_s *ali, 
				      int alignment, int sweep, int traceback, int voutput, struct end_s *othends);
extern void          ViterbiOTHDiagMatrix(FILE *ofp, double **mt, SQINFO sqinfoX, int *seqX, SQINFO sqinfoY, 
					  int *seqY, int start, int L, struct othmodel_s *oth, 
					  struct othdpd_s *othdp);
extern void          ViterbiOTHFullMatrix(FILE *ofp, double **mt, SQINFO sqinfoX, int *seqX, SQINFO sqinfoY, 
					  int *seqY, int startX, int startY, int Lx, int Ly, int Lw,
					  struct othmodel_s *oth, struct othdpf_s *othdp);
extern void          ViterbiOTHFullVector(FILE *ofp, double *v, SQINFO sqinfoX, int *seqX, SQINFO sqinfoY, 
					  int *seqY, int startX, int startY, int Lx, int Ly, 
					  struct othmodel_s *oth, struct othdpf_s *othdp);
extern void          ViterbiOTHDiagVector(FILE *ofp, double *v, SQINFO sqinfoX, int *seqX, SQINFO sqinfoY, 
					  int *seqY, int start, int L, struct othmodel_s *oth, 
					  struct othdpd_s *othdp);

/* from othmodel.c:
 */
extern struct othmodel_s *AllocOTHModel(void);
extern void    CheckDoubleProb(double **pdouble, int dimx, int dimy);
extern void    CheckOtherProbs(struct othmodel_s *oth);
extern void    CheckSingleProb(double *psingle, int size);
extern void    CheckSingleLog2Prob(double *psingle, int size);
extern void    ConstructMemOTHProbs(double *mutpxy, double *mem);
extern void    ConstructOTHModel(double *mutpxy, struct othparam_s param, double tfactor, 
				 struct othmodel_s *oth, int pedantic);
extern void    ConstructXemOTHProbs(double *mem, double *xem);
extern void    ConstructYemOTHProbs(double *mem, double *yem);
extern void    FreeOTHModel(struct othmodel_s *oth);
extern double *Joint_From_Condi(FILE *ofp, double *Q, double *pm, int L, int verbose);
extern void    OTHToLog2(struct othmodel_s *other);
extern void    OTHLog2ToOdds(struct othmodel_s *oth, struct nullmodel_s *null);
extern void    PrintMutProbs(double *mutpxy, double *pnull);
extern void    PrintMut5Probs(double *mut5pxy, double *pnull);
extern void    PrintOTHModel(struct othmodel_s *oth);

/* from otherscorealing.c:
 */
extern double        ScoreWithOTH_ungapped(int *sX, int *sY, int L, struct othmodel_s *oth);
extern struct sc2_s *ScoreWithOTH2(FILE *ofp, int *seqX, int *seqY, int start, int L,
				   struct othmodel_s *oth, int traceback);
extern double        ScoreWithOTH(FILE *ofp, int *seqX, int *seqY, int start, int L,
				  struct othmodel_s *oth, int traceback);
extern double        ScoreWithOTHMatrix(int *sX, int *sY, int j, struct othmodel_s *oth);

/* from posterior.c:
 */
extern double Posterior(double sc1, double sc2, double sc3);
extern double PosteriorLog2(double sc1, double sc2, double sc3);

/* from rnacreatealing.c:
 */
extern void SimulateNUSSequences(struct rnamodel_s *rna, int *ct, int L, 
				 char **ret_sX, char **ret_sY);
extern void SimulateRNAAlign(FILE *ofp, SQINFO *sqinfoX, int *seqX, SQINFO *sqinfoY, int *seqY, int start, 
			     int L, struct rnamodel_s *rna, struct nullmodel_s *null, 
			     struct ali_s *ali, int *ct, int traceback, int alignment, char *string_name);
extern void SimulateRNASequence(FILE *ofp, int *seq, SQINFO *sqinfoX, int *seqX, SQINFO *sqinfoY, 
				int *seqY, int start, int L, struct rnamodel_s *rna, 
				struct nullmodel_s *null, struct ali_s *ali, int *ct, int traceback, 
				int alignment, char *string_name);
/* from rnadp.c:
 */
extern struct rnadp_s *AllocDpDiagRNA(int L);
extern struct rnadp_s *AllocDpFullRNA(int L);
extern struct sc2_s   *ForwardRNADiag(FILE *ofp, SQINFO sqinfoX, int *seqX, SQINFO sqinfoY, int *seqY, int start, int L, 
				      struct model_s *model, struct dpd_s *dp, struct rnascfg_s *mx, 
				      struct dpsc3_s *sc, int *ret_scfg_status, int *ret_scfgrv_status, 
				      int cyk, int logodds, int do_nus);
extern void            FreeDpRNA(struct rnadp_s *dp);
extern void            PatternDpRNA(int L, struct rnadp_s *rnadp);
extern double          ViterbiRNAFull(FILE *ofp, SQINFO sqinfoX, int *seqX, SQINFO sqinfoY, 
				      int *seqY, int startX, int Lx, int startY, int Ly, 
				      struct rnamodel_s *rna, struct dpf_s *dp, struct rnascfg_s *mx, 
				      struct ali_s *ali, int alignment, int traceback);
extern struct sc2_s   *ViterbiRNADiag(FILE *ofp, SQINFO sqinfoX, int *seqX, SQINFO sqinfoY, int *seqY, 
				      int start, int L, struct model_s *model, struct dpd_s *dp, 
				      struct rnascfg_s *mx, struct ali_s *ali, int *ret_scfg_status, int *ret_scfgrv_status, 
				      int alignment, int cyk, int logoods, int do_nus, int traceback, 
				      struct end3_s *ends);

/* from rnaio.c:
 */
extern int KHS2ct(char *ss, int len, int allow_pseudoknots, int **ret_ct);
extern int VerifyKHS(char *name, char *ss, int wordy);

/* from rnamodel.c:
 */
extern struct nusmodel_s *AllocNUSModel(void);
extern struct pi2model_s *AllocPI2Model(void);
extern struct rnamodel_s *AllocRNAModel(void);
extern void     CheckRNAProbs(struct rnamodel_s *rna);
extern void     ConstructRNAModel(double *mutprob, double *mut5prob, double ***cfg_node, 
				  double *pair5prob, double tfactor, struct rnamodel_s *rna, struct nullmodel_s *null, 
				  double *freqX, double *freqY, int changefreq, int pedantic);
extern void     FreeNUSModel(struct nusmodel_s *nus);
extern void     FreePI2Model(struct pi2model_s *pi2);
extern void     FreeRNAModel(struct rnamodel_s *rna);
extern void     NUSToLog2(struct nusmodel_s *nus);
extern void     NUSLog2ToOdds(struct nusmodel_s *nus, struct nullmodel_s *null);
extern void     PI2ToLog2(struct pi2model_s *pi2, struct nullmodel_s *null);
extern void     PI2Log2ToOdds(struct pi2model_s *pi2, struct nullmodel_s *null);
extern void     PatternRNAModel(struct rnamodel_s *rna);
extern void     RNAToLog2(struct rnamodel_s *rna, struct nullmodel_s *null);
extern void     RNALog2ToOdds(struct rnamodel_s *rna, struct nullmodel_s *null);
extern void     PrintNUSModel(struct nusmodel_s *nus);
extern void     PrintPI2Model(struct pi2model_s *pi2);
extern void     PrintRNAModel(struct rnamodel_s *rna);

/* from rnaoutput.c: conversion of internal data structures into
 *                   more user-friendly forms
 */
extern void CompareRNAStructures(FILE *ofp, int start, int L, char *ss_true, char *ss);
extern void Ct2KHS_ER(int start, int L, int *ct, char **ret_cc);
extern int  IsRNAComplement(char sym1, char sym2, int allow_gu);
extern void RNAPairs(SQINFO sqinfo, int *ss, int format, float *ret_pairs, float *ret_cykpairs);
extern void TraceRNA(struct tracekn_s *tr, int rlen, int lend, int watsoncrick, char **ret_ss, char **ret_cc) ;
extern void Tracenrn(struct tracekn_s *tr, int rlen, int lend, int watsoncrick, char **ret_ss, char **ret_cc);
                     

/* from rnascorealing.c:
 */
extern double        ScoreWithNUS_ungapped(int *sX, int *sY, int L, struct nusmodel_s *nus, int traceback);
extern struct sc2_s *ScoreWithRNA(FILE *ofp, SQINFO sqinfoX, int *sX, SQINFO sqinfoY, int *sY, int start, int L, 
				  struct model_s *model, struct othdpd_s *othdp, struct rnascfg_s *mx, 
				  int *ret_scfg_status, int *ret_scfgrv_status, int cyk, int logodds, int do_nus, int traceback);

/* from rnascfg.c:
 */
extern struct rnamtx_s  *AllocRNAMtx(int L);
extern struct rnascfg_s *AllocScfgRNA(int L, int rnass);
extern void     BestV (int *sX, int *sY, struct pi2model_s *pi2, int start, int L, int j, int d, 
		       struct rnamtx_s *mtx, double *vp);
extern void     BestW (int *sX, int *sY, struct pi2model_s *pi2, int start, int L, int j, int d, 
		       struct rnamtx_s *mtx);
extern void     BestWB(int *sX, int *sY, struct pi2model_s *pi2, int start, int L, int j, int d, 
		       struct rnamtx_s *mtx);
extern void     CalPairs (int *sX, int *sY, int start, int L, struct pi2model_s *pi2, 
			  struct rnamtx_s *in, struct rnamtx_s *ou, double ***pp, double **ps, struct end_s *rnaends);
extern int      CYKRNA(FILE *ofp, SQINFO sqinfoX, int *sX, SQINFO sqinfoY, int *sY, int start, int L, 
		       struct rnamodel_s *rna, struct nullmodel_s *null, struct othdpd_s *othdp, 
		       struct rnamtx_s *mtx, double *vp, 
		       int logoods, int do_nus, int traceback);
extern void     CYKTracebackRNA(FILE *ofp, SQINFO sqinfoX, int *sX, SQINFO sqinfoY, int *sY, int start, int L,
				struct rnamodel_s *rna, struct rnascfg_s *mx, 
				int do_nus, int logodds, int traceback, struct end_s *rnaends, int isrnarev);
extern void     FillV (int *sX, int *sY, struct pi2model_s *pi2, int start, int L, int j, int d, 
		       struct rnamtx_s *mtx, double *vp, double *sc);
extern void     FillVO(int *sX, int *sY, struct pi2model_s *pi2, int start, int L, int j, int d, 
		       double *vp, double **wox, double **wbox, double **vox, double *sc, struct end_s *rnaends);
extern void     FillW (int *sX, int *sY, struct pi2model_s *pi2, int start, int L, int j, int d, 
		       struct rnamtx_s *mtx, double *sc);
extern void     FillWO(int *sX, int *sY, struct pi2model_s *pi2, int start, int L, int j, int d, 
		       double **wox, double **wx, double *sc, struct end_s *rnaends);
extern void     FillWB(int *sX, int *sY, struct pi2model_s *pi2, int start, int L, int j, int d, 
		       struct rnamtx_s *mtx, double *sc);
extern void     FillWBO(int *sX, int *sY, struct pi2model_s *pi2, int start, int L, int j, int d, double **wbox,
			double **wbx, double **vox, double *sc, struct end_s *rnaends);
extern void     FreeRNAMtx(struct rnamtx_s *mtx);
extern void     FreeScfgRNA(struct rnascfg_s *mx, int rnass);
extern int      InsideRNA(FILE *ofp, SQINFO sqinfoX, int *sX, SQINFO sqinfoY, int *sY, 
			  int start, int L, struct rnamodel_s *rna,  struct nullmodel_s *null, 
			  struct othdpd_s *othdp, struct rnamtx_s *mtx, double *sc, double *vp, 
			  int do_nus, int logodds);
extern int      OutsideRNA(FILE *ofp, SQINFO sqinfoX, int *sX, SQINFO sqinfoY, int *sY, int start, int L,
			   struct rnamodel_s *rna, struct rnamtx_s *in, struct rnamtx_s *ou, double *vp, double *sc,
			   int do_nus, int logodds, struct end_s *rnaends);
extern void     Tr_V(FILE *ofp, int *sX, int *sY, struct pi2model_s *pi2, int start, int L, int j, int d, 
		     struct rnamtx_s *mtx, double *vp,
		     int *flag, struct tracekn_s *curr_tr, struct traceknstack_s *dolist, int traceback, struct end_s *rnaends);
extern void     Tr_W(FILE *ofp, int *sX, int *sY, struct pi2model_s *pi2, int start, int L, int j, int d, struct rnamtx_s *mtx, 
		     int *flag, struct tracekn_s *curr_tr, struct traceknstack_s *dolist, int traceback, struct end_s *rnaends);
extern void     Tr_WB(FILE *ofp, int *sX, int *sY, struct pi2model_s *pi2, int start, int L, int j, int d, struct rnamtx_s *mtx, 
		      int *flag, struct tracekn_s *curr_tr, struct traceknstack_s *dolist, int traceback, struct end_s *rnaends);
extern void     Trk_RNA(FILE *ofp, SQINFO sqinfoX, int *sX, SQINFO sqinfoY, int *sY, int start, int L,  int j, int d,
			struct pi2model_s *pi2, struct rnamtx_s *mtx, double *vp,
			struct tracekn_s **ret_trace, int traceback, struct end_s *rnaends);
extern void     PatternVec(int L, double *vec);
extern void     PatternMtx(int L, double **m);
extern void     PatternRNAMtx(int L, struct rnamtx_s *rnamtx);
extern void     PatternScfgRNA(int L, struct rnascfg_s *mx, int rnass, int rvrna);
extern void     PosteriorRNA(FILE *ofp, SQINFO sqinfoX, int *sX, SQINFO sqinfoY, int *sY, int start, int L,
			     struct rnamodel_s *rna, struct rnascfg_s *mx, 
			     int do_nus, int logodds, int traceback, struct end_s *rnaends, int isrnarev);



/* from scorewithmodels.c:
 */
extern struct end3_s   *AllocEnds();
extern struct sc3_s    *AllocSc3();
extern struct scores_s *AllocScores();
extern void             FreeEnds(struct end3_s *ends);
extern void             FreeSc3(struct sc3_s *sc3);
extern void             FreeScores(struct scores_s *sc);
extern void             PrintScores(FILE *ofp, struct sc3_s *sc, struct end3_s *ends);
extern void             ScoreWithModels(FILE *ofp, 
					SQINFO sqinfoX, int *isegX, int *iseqX, 
					SQINFO sqinfoY, int *isegY, int *iseqY, 
					int start, int leg, int Lmax, 
					struct dos_s dos, 
					struct model_s *model, 
					struct dpd_s *dpd, struct dpf_s *dpf, 
					struct rnascfg_s *mx, 
					struct scores_s *sc, 
					struct ali_s *ali,  
					int alignment, int cyk, int logodds, int rnass, int shuffle, int sweep, int traceback,
					int verbose);
extern char            *WinnerScore(struct sc3_s *sc);

/* from seqio.c:
 */
extern void     AlignStat(FILE *ofp, SQINFO *sqinfoX, int *seqX, SQINFO *sqinfoY, int *seqY, int len, 
			  double *id, double *gap, double *mut);
extern double *AllocBaseFreq(void);
extern void    AllocCharSeq(int Lmax, char **ret_seq);
extern void    AllocCharSeqs(int Lmax, char **ret_seqX, char **ret_seqY);
extern void    AllocIntSeq(int Lmax, int **ret_iseq);
extern void    AllocIntSeqs(int Lmax, int **ret_iseqX, int **ret_iseqY);
extern void    AllocCt(int Lmax, int **ret_ct);
extern void    ArrangeSequence(FILE *ofp, SQINFO *sqinfo, int format, char *seq, int **ret_ct, 
			       int allow_pseudoknots, int verbose);
extern void    ArrangeGapSequence(FILE *ofp, SQINFO *sqinfo, int format, char *seq, int **ret_ct, 
				  int allow_pseudoknots, int verbose);
extern void    BaseComp(FILE *ofp, int *s, int j, int d, double *basefreq);
extern void    BaseCount(FILE *ofp, int *s, int j, int d, double *basecount);
extern void    BaseCompChar(FILE *ofp, char *s, int j, int d, double *basefreq);
extern void    BaseCompNorm(int nbases, double *basefreq);
extern void    CalculatePairs(FILE *outf, char *seq, SQINFO sqinfo, int *ss);
extern void    CheckMaxLength(SQFILE *sqfp, int format, long int *ret_Lmax, int *ret_nseqs, long int maxhitlen);
extern void    Count2Freq(FILE *ofp, int dim, double *basecount);
extern char   *DedigitizeSequence(int *dsq, int start, int L);
extern int    *DigitizeSequence(char *seq, int L);
extern void    DupIntAseq(int *seq, int *copyseq, int len);
extern void    DupIntSeq(int *iseq, int *copyseq, int j, int d);
extern char   *DupSeq(char *seq, int j, int d);
extern void    IntizeGapAsequence(char *seq, int start, int len, int *iseq, int verbose);
extern void    IntizeGapSequence(char *seq, int len, int **ret_iseq);
extern void    IntizeSequence(char *seq, int len, int **ret_iseq);
extern void    PatternIntSeqs(int Lmax, int *iseqX, int *iseqY);
extern double  PerIdSeqs(int *iseqX, int *iseqY, int j, int d);
extern int     PosNoGaps(int *seq, int pos);
extern void    PrintAlign(FILE *ofp, SQINFO sqinfoX, SQINFO sqinfoY, int start, int L, 
			  struct ali_s *ali);
extern void    PrintNewAlign(FILE *ofp, SQINFO *sqinfoX, SQINFO *sqinfoY, int start, int L, 
			     struct ali_s *ali, char *string_name);
extern void    PrintCtAlign(FILE *ofp, SQINFO *sqinfoX, SQINFO *sqinfoY, int start, int L, 
			    struct ali_s *ali, int *ct, char *string_name);
extern void    PrintCtSeqs(FILE *ofp, SQINFO *sqinfoX, int *sX, SQINFO *sqinfoY, int *sY, int start, int L, char *ss);
extern void    PrintSSseqs(FILE *ofp, SQINFO *sqinfoX, int *seqX, SQINFO *sqinfoY, int *seqY, 
			   int start, int L, char *ss, char *cc);
extern void    Print2Seqs(FILE *ofp, SQINFO *sqinfoX, int *sX, SQINFO *sqinfoY, int *sY, int start, int L);
extern void    PrintBanner(FILE *ofp, char *pamfile, float tfactor, char *rnafile, char *seqfile, 
			   char *seqfile_b, int allow_pseudoknots, int nseqs, long int Lmax, long int maxhitlen, 
			    int minhitlen, int sweep, int window, int slide);
extern void    PrintSeq(FILE *ofp, SQINFO *sqinfo, int leg, char *seg);
extern void    PrintIntSeq(FILE *ofp, SQINFO *sqinfo, int leg, int *iseg);
extern void    PrintAlignSequences(FILE *ofp, SQINFO *sqinfoX, SQINFO *sqinfoY, char *string_name, 
				   double tfactor, int start, int len, int *seqX, int *seqY);
extern void    PrintIntSequence(FILE *ofp, SQINFO *sqinfo, char *name, int start, int len, int *seq);
extern void    PrintHitBanner(FILE *ofp, SQINFO sqinfoX, SQINFO sqinfoY, int nseqs, int shuffle, int strand);
extern void    PrintNull(FILE *ofp, struct nullmodel_s *null, double nullsc, double nullscrev);
extern void    RemoveGaps(int *s, int Lmax, int *len, int verbose);
extern void    RemoveJointGaps(int lenX, int *seqX, int lenY, int *seqY, int *len, int sweep);
extern void    RevComp (int *rev, int *seq, int L);
extern void    RevCompChar (char *rev, char *seq, int L);
extern void    ShuffleSequence(char *seq, int len, int endpos, int verbose);
extern void    ShuffleIntSequence(int *iseq, int len, int endpos, int verbose);
extern void    Shuffle2IntSequences(int *iseqX, int *iseqY, int len, int endposX, int endposY, int verbose);
extern void    StripDegeneracy(char *seq, int *len);
extern void    StripSomeDegeneracy(char *seq, int len);
extern void    ToFold(SQINFO sqinfo, int pos_init, int pos_end, int *ret_begin, 
		      int *ret_end, int *ret_len);
extern void    WriteSeqinfo(FILE *outf, SQINFO *sqinfo, int nfile, int nseq, int nbases);
extern int     WriteSeqkn(FILE *ofp, char *seq, SQINFO *sqinfo, int *ret_ss);


/* from trace.c
 */
extern struct tracer_s      *InitTracer(void);
extern struct tracerstack_s *InitTracerstack(void);
extern struct tracer_s      *AttachTracer(struct tracer_s *parent, int emit, int type);
extern struct tracerstack_s *InitTracerstack(void);
extern struct tracer_s      *PopTracerstack(struct tracerstack_s *stack);
extern void                  FreeTracer(struct tracer_s *tr);
extern void                  DeleteTracernode(struct tracer_s *oldtr);
extern void                  PushTracerstack(struct tracerstack_s *stack, struct tracer_s *node);
extern void                  FreeTracerstack(struct tracerstack_s *stack);

/* from traceSCFG.c
 */
extern struct trace_s   *InitTrace(void);
extern struct tracekn_s *InitTracekn(void);
extern struct trace_s   *AttachTrace(struct trace_s *parent, int emitr, int emitl, int type);
extern struct tracekn_s *AttachTracekn(struct tracekn_s *parent, 
				       int emiti, int emitj, int emitk, int emitl, 
                                       int type, int node);
extern struct tracestack_s   *InitTracestack(void);
extern struct traceknstack_s *InitTraceknstack(void);
extern struct trace_s        *PopTracestack(struct tracestack_s *stack);
extern struct tracekn_s      *PopTraceknstack(struct traceknstack_s *stack);
extern void   FreeTrace(struct trace_s *tr);
extern void   FreeTracekn(struct tracekn_s *tr);
extern void   DeleteTracenode(struct trace_s *oldtr);
extern void   DeleteTraceknnode(struct tracekn_s *oldtr);
extern void   PushTracestack(struct tracestack_s *stack, struct trace_s *node);
extern void   PushTraceknstack(struct traceknstack_s *stack, struct tracekn_s *node);
extern void   FreeTracestack(struct tracestack_s *stack);
extern void   FreeTraceknstack(struct traceknstack_s *stack);

/* from tying.c:
 */
extern void    BootstrapConfidence(double **cfg, double *counts, int **enslave, int nboot, double conf,
				   double ***ret_high, double ***ret_low);
extern void    CountFreeParameters(int **enslave, int *ret_free, int *ret_nonzero, int allow_pseudoknots);
extern double *CountsPerState(double **cfg);
extern int   **EnslaveTrans(void);
extern void    TieCounts(double **cfg, int **enslave);

#endif /*FUNCSH_INCLUDED*/



