/************************************************************
 * QRNA - Comparative analysis of biological sequences 
 *        with pair hidden Markov models and pair stochastic context-free grammars
 * Copyright (C) 2000-Howard Hughes Medical Institute
 * All Rights Reserved
 * 
 *     This source code is distributed under the terms of the
 *     GNU General Public License. See the files COPYING and LICENSE
 *     for details.
 ************************************************************/

/* globals.h
 * Global variable definitions. 
 * This file may only be included in a main() .c file.
 *
 * ER, Fri May 28 11:33:45 CDT 1999 [St. Louis]
 * 
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <limits.h>

#include "squid.h"

/* genome "worm rules" 
 */
#define accuracy 0.99
		
/* Useful macros 
 */
#define LN2         0.69314718056
#define LN2INV      1.44269504089
#define LOG2(x)     ((x) == 0.0 ? -9999. : log(x) * LN2INV)
#define EXP2(x)     (exp((x) * LN2 )) 
#define INTSCALE    100.0
#define BIGINT      9999999	          /* prohibition in viterbi.c without overflow              */
#define BIGFLOAT    999.999	          /* prohibition in viterbi.c without overflow              */
#define MAXRNALOOP  200                   /* maximum loop size allowed == d == (MAXRNALOOP-1)       */
#define PROHIBIT    50.0
#define GASCONST    8.31441               /* boltzman factor (J mol^{-1} K^{-1})                    */
#define GASCAL      (GASCONST*0.000239)   /* in Kcal: 1 J= 0.239 cal                                */
#define BETAINV     (GASCAL*310.15)       /* 1/beta = kT [Kcal/mol) (T=310 K)                       */
#define BETA        (1.0/BETAINV)         /* beta = 1/kT (T=310 K)                                  */

extern char Alphabet[];
extern char DNAAlphabet[];
extern char SSAlphabet[];


#define CODON(x1,x2,x3)   ((x1)*16 + (x2)*4 + (x3))  /* Lookup index of a codon with bases x1x2x3 */
#define CODON5(x1,x2,x3)  ((x1)*25 + (x2)*5 + (x3))  /* Lookup index of a codon with gaps         */
#define idx(x,y)          ((x)*4+(y))                /* Lookup index of a mut prob xy             */
#define idx5(x,y)         ((x)*5+(y))                /* Lookup index of a mut5 prob xy            */

#define SYMIDX(x)    (strchr(Alphabet, (x)) - Alphabet)
#define DNAIDX(x)    (strchr(DNAAlphabet, (x)) - DNAAlphabet)

#define sreLOG2(x)   ((x) > 0 ? log(x) * 1.44269504 : -9999.)
#define sreEXP2(x)   (exp((x) * 0.69314718 )) 


#define sre_isascii(c)   (!((c) & ~0177))














